@@ -181,7 +181,6 @@ class Irradiance():
 
     data['G_Rear'] = G_Rear
     data['Total_G'] = G_Rear + G_front
-    data[['GHI', 'DHI','DNI','G_Front','Total_G','G_Rear']] = data[['GHI', 'DHI','DNI','G_Front','Total_G','G_Rear']].clip(lower=0)
     data['Total_G'] = data['Total_G'].round(0)
     #data = data.drop(['Declination', 'Hour angle','Rb_front','Rb_rear','Solar Zenith angle','GHI','DNI','DHI','2m Air Temperature','10m Wind speed'], axis=1)
     
@@ -198,6 +197,9 @@ class Irradiance():
     new_df = new_df.interpolate(method='polynomial', order=2)
     data = new_df
 
+    data.drop(data.tail(n).index,inplace=True) # drop last n rows
+    data[['GHI', 'DHI','DNI','G_Front','Total_G','G_Rear']] = data[['GHI', 'DHI','DNI','G_Front','Total_G','G_Rear']].clip(lower=0)
+
     return inputs ,data, metadata
 
 
@@ -453,11 +455,6 @@ class Irradiance():
     return GR_reflected
 
 
-    return E_rear
-
-
-
-
 #
 #Auxiliry methods:
 #
